// This is a script with examples for adding stuff to be displayed via MapUtils, SkInfo
// or SOCGen. Similiar to how modularity was done for Windows for SRB2, listing the
// extension's name is done by calling the functions a special way, returning
// a string that can be safely discarded if not needed.

// In this example, we check if Daytona Kart is loaded and a map has times set for it.
// Listing it is MUDaytonaKart(nil, true).
local function MUDaytonaKartCompat(mapid, name, p)
	if name then return "Daytona Kart Compatibility" end
	if daytonakart and maptimes and mapheaderinfo[mapid].arcadetime != nil and mapheaderinfo[mapid].arcadebonus != nil then
		return "(Daytona Kart)" // Return a string for our presence.
	end
end
table.insert(MUtils.specialmapinfo, MUDaytonaKartCompat)

// In this example, we check if the skin is using "Simple AnimaL".
// Listing it is SISimpleAnimaL(nil, true).
local function SISimpleAnimaL(intname, name, p)
	if name then return "Simple AnimaL Detection" end
	if _G["SIMPLE_ANIMAL_VERSION"] and _G["SIMPLE_ANIMAL_DEFINITIONS"]
	and _G["SIMPLE_ANIMAL_DEFINITIONS"][intname] then
		return "(Simple AnimaL)" // Make our presence known!
	end
end
table.insert(SkInfo.specialskinattrib, SISimpleAnimaL)

// You can return a single string (Lua.testval = 1) or a table of strings if there's
// multiple "Lua.customval" for SOCGen to use. In this example, we'll return a table
// of strings which will be unpacked by the script.
// Listing it is SOCGenDaytonaKart(nil, true).
local function SOCGenDaytonaKart(mapid, name, p)
	if name then return "Daytona Kart Detection" end
	local outtable = {}
	if daytonakart and maptimes then
		// The "and" is important here - we want to see if the value is neither nil OR empty.
		if mapheaderinfo[mapid].arcadetime != nil and mapheaderinfo[mapid].arcadetime != "" then
			table.insert(outtable, "lua.arcadetime = "..mapheaderinfo[mapid].arcadetime)
		end
		if mapheaderinfo[mapid].arcadebonus != nil and mapheaderinfo[mapid].arcadebonus != "" then
			table.insert(outtable, "lua.arcadebonus = "..mapheaderinfo[mapid].arcadebonus)
		end
		if mapheaderinfo[mapid].noextension != nil and mapheaderinfo[mapid].noextension != "" then
			table.insert(outtable, "lua.noextension = "..mapheaderinfo[mapid].noextension)
		end
		if mapheaderinfo[mapid].arcadeboss != nil and mapheaderinfo[mapid].arcadeboss != "" then
			table.insert(outtable, "lua.arcadeboss = "..mapheaderinfo[mapid].arcadeboss)
		end
		if #outtable > 0 then return outtable else return end // If there's actually something, return the table.
	end
end
table.insert(SOCGen.mapheaderextra, SOCGenDaytonaKart)